from os import PathLike
from pydantic import BaseModel, Field
import uuid
from typing import Type, TypeVar, Generic, Optional
import rocksdbpy


class IdModel(BaseModel):
    id: str = Field(default='')


T = TypeVar('T', bound=IdModel)


class KytoDbCollection(Generic[T]):
    def __init__(self, db: rocksdbpy.RocksDB, model: Type[T], collection_name: str):
        self.db = db
        self.model = model
        self.collection_name = collection_name

    def add(self, obj: T) -> str:
        if not obj.id:
            obj.id = str(uuid.uuid4())
        key = self._construct_key(obj.id)
        value = obj.model_dump_json().encode('utf-8')
        self.db.set(key, value)
        return obj.id

    def get(self, obj_id: str) -> Optional[T]:
        key = self._construct_key(obj_id)
        value = self.db.get(key)
        if value is not None:
            return self.model.model_validate_json(value)
        return None

    def update(self, obj: T):
        if obj.id == '':
            raise ValueError('ID must not be empty during update')
        key = self._construct_key(obj.id)
        value = obj.model_dump_json().encode('utf-8')
        self.db.set(key, value)

    def _construct_key(self, obj_id: str) -> bytes:
        return f'{self.collection_name}:{self.model.__name__}:{obj_id}'.encode('utf-8')


class KytoDbClient:
    def __init__(self, db_path: str | PathLike[str]):
        opts = rocksdbpy.Option()
        opts.create_if_missing(True)

        self.db = rocksdbpy.open(str(db_path), opts)

    def collection(self, model: Type[T], collection_name: str) -> KytoDbCollection[T]:
        return KytoDbCollection(self.db, model, collection_name)

    def close(self):
        self.db.close()



import pytest
import tempfile
import uuid
from kytodb import KytoDbClient, IdModel


class MyModel(IdModel):
    name: str


@pytest.fixture
def temp_db():
    with tempfile.TemporaryDirectory() as temp_dir:
        client = KytoDbClient(temp_dir)
        yield client
        client.close()


@pytest.fixture
def collection(temp_db):
    return temp_db.collection(MyModel, 'test_collection')


def test_add_generates_id_and_stores(collection):
    obj = MyModel(name='Test Object')
    new_id = collection.add(obj)
    retrieved = collection.get(new_id)
    assert obj.id == new_id
    assert retrieved == obj


def test_add_raises_if_id_not_empty(collection):
    obj = MyModel(id='123', name='Test Object')
    obj_id = collection.add(obj)
    assert obj_id == '123'


def test_get_returns_object_if_found(collection):
    obj = MyModel(name='Test Object')
    new_id = collection.add(obj)
    retrieved = collection.get(new_id)
    assert retrieved == obj


def test_get_returns_none_if_not_found(collection):
    result = collection.get(str(uuid.uuid4()))
    assert result is None
